from symbl.Conversations import Conversation
from symbl.AuthenticationToken import get_api_client
from symbl_rest import AsyncApi as async_api_rest

def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if 'credentials' in kw:
            credentials = kw['credentials']

        api_client = get_api_client(credentials)
        self.__async_api_rest = async_api_rest(api_client)

        return function(*args, **kw)
    
    return wrapper
    
class Text():

    def __init__(self, api_client=None):
        '''
            It will initialize the Analysis class
            with the object of Initialize Class
        '''
        self.__async_api_rest = async_api_rest(api_client)

    @initialize_api_client
    def process(self, payload : dict, credentials=None, wait: bool = True):
        '''
            Text payload to be analyzed
            returns Conversation object
        '''
        
        if payload == None:
            raise ValueError("Please enter a valid payload.")

        response = self.__async_api_rest.add_text(body=payload)
        print("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)

    @initialize_api_client
    def append(self, payload:dict, conversation_id:str, credentials=None, wait: bool = True):
        '''
            Text payload to be appended
            returns Conversation object
        '''
        if payload == None:
            raise ValueError("Text Payload can not be None")

        if conversation_id == None or len(conversation_id) == 0:
            raise ValueError("Please enter a valid conversationId")

        response = self.__async_api_rest.append_text(conversation_id, body=payload)
        print("Job with jobId {} for conversationId {} started".format(response.job_id, response.conversation_id))

        return Conversation(response.conversation_id, response.job_id, wait=wait, credentials=credentials)


from symbl.AuthenticationToken import get_api_client
from symbl_rest import ConversationsApi as conversations_api_rest

class ConversationsApi():

    def __init__(self, api_client=None):
        '''
            It will initialize the ConversationsApi class
        '''
        
        self.api_client = api_client
        self.conversations_api_rest = conversations_api_rest(api_client)

    def initialize_api_client(function):
        def wrapper(*args, **kw):
            credentials = None
            self = args[0]
            
            if 'credentials' in kw:
                credentials = kw['credentials']

            api_client = get_api_client(credentials)
            self.api_client = api_client
            self.async_api_rest = conversations_api_rest(api_client)

            return function(*args, **kw)
        
        return wrapper

    @initialize_api_client
    def get_action_items(self, conversation_id, credentials=None):
        return self.conversations_api_rest.get_action_items_by_conversation_id(conversation_id)

    @initialize_api_client
    def get_follow_ups(self, conversation_id, credentials=None ):  
        return self.conversations_api_rest.get_follow_ups_by_conversation_id(conversation_id)

    @initialize_api_client
    def get_insights(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_insights_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_members(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_members_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_messages(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_messages_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_questions(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_questions_by_conversation_id(conversation_id)
  
    @initialize_api_client      
    def get_topics(self, conversation_id, credentials=None):  
        return self.conversations_api_rest.get_topics_by_conversation_id(conversation_id)



from symbl.AuthenticationToken import get_api_client
from symbl.jobs_api.JobStatus import JobStatus
from symbl_rest import JobsApi

import threading
import time


def initialize_api_client(function):
    def wrapper(*args, **kw):
        credentials = None
        self = args[0]
        
        if not hasattr(self,'__jobs_api'):
            if 'credentials' in kw:
                credentials = kw['credentials']
            api_client = get_api_client(credentials)

            self.__jobs_api = JobsApi(api_client)

        function(*args, **kw)
    
    return wrapper
class Job():

    __INTERVAL_TIME_IN_SECONDS = 5  ## in seconds

    def __init__(self, job_id: str, conversation_id:str, wait=True):

        self.__job_id = job_id
        self.__conversation_id = conversation_id
        self.__success_func = None
        self.__jobs_api = JobsApi()
        self.__error_func = None
        self.__job_status = JobStatus.IN_PROGRESS
        self.__wait = wait
    
    def getConversationId(self):
        return self.__conversation_id
    
    def getJobStatus(self):
        return self.__job_status.value

    def on_complete(self, func):
        self.__success_func = func
        return self

    def on_error(self, func):
        self.__error_func = func
        return self
    

    @initialize_api_client
    def __fetch_current_job_status(self, credentials=None):
        if self.__jobs_api != None:
            response = self.__jobs_api.get_job_status(self.__job_id)
            self.__job_status = JobStatus(response.status)
        else:
            raise ValueError("Job object not initialized correctly. Please contact administrator.")

    def syncronous_monitor_job(self, conversation, interval=None, wait=True, credentials=None):
        if interval is None:
            interval = self.__INTERVAL_TIME_IN_SECONDS
        
        while self.__job_status != JobStatus.COMPLETED and self.__job_status != JobStatus.FAILED:
            self.__fetch_current_job_status(credentials=credentials)
            print("Fetching latest status of job {0}, current status is {1}".format(self.__job_id, self.__job_status.value))
            time.sleep(interval)

        if self.__job_status == JobStatus.COMPLETED and self.__success_func != None:
            self.__success_func(conversation)

        elif self.__error_func != None:
            self.__error_func(conversation)

    
    def monitor_job(self, conversation, interval=None, wait=True, credentials=None):
        if wait:
            self.syncronous_monitor_job(conversation, interval, wait, credentials)
        else:
            thread = threading.Thread(target=self.syncronous_monitor_job, args=(conversation, interval, wait, credentials))
            thread.start()